#ifndef BOSON_LEXER_H
#define BOSON_LEXER_H

#include <stdio.h>

struct ast_statement;

enum lex_token {
	// Keywords
	T_AND,
	T_BREAK,
	T_CONTINUE,
	T_ELIF,
	T_ELSE,
	T_ENDFOREACH,
	T_ENDIF,
	T_FALSE,
	T_FOREACH,
	T_IF,
	T_IN,
	T_NOT,
	T_OR,
	T_TRUE,
	T_LAST_KEYWORD = T_TRUE,

	// Operators
	T_ASSIGN,
	T_COLON,
	T_COMMA,
	T_DOT,
	T_EQ,
	T_GEQ,
	T_GT,
	T_LBRACK,
	T_LCURL,
	T_LEQ,
	T_LPAREN,
	T_LT,
	T_MINEQ,
	T_MINUS,
	T_MODEQ,
	T_MODULO,
	T_NEQ,
	T_PLUS,
	T_PLUSEQ,
	T_QUESTION,
	T_RBRACK,
	T_RCURL,
	T_RPAREN,
	T_SLASH,
	T_SLASHEQ,
	T_STAR,
	T_STAREQ,
	T_LAST_OP = T_STAREQ,

	// Data
	T_IDENTIFIER,
	T_NUMBER,
	T_STRING,

	// Magic
	T_EOF,
	T_EOL,
	T_ERROR,

};

struct location {
	const char *path;
	int lineno, colno;
};

struct token {
	struct location loc;
	enum lex_token type;
	struct {
		size_t n;
		char *data;
	} string;
};

struct lexer {
	FILE *file;
	struct location loc;
	struct token prev;
	int cur;
};

enum lex_token lex(struct lexer *, struct token *);
enum lex_token unlex(struct lexer *, struct token *);
struct ast_statement *parse(struct lexer *);

void lexer_init(struct lexer *, const char *);
void lexer_finish(struct lexer *);
void token_finish(struct token *);

const char *token_str(const struct token *);
const char *lex_token_str(enum lex_token);
const char *loc_str(const struct location *);

#endif // BOSON_LEXER_H
